@@ -1,9 +1,9 @@
 """Configuration for custom attributes generator."""
 
 STATIC_IDS_JSON_FILE = "special_attributes.json"
-JSON_DATA_FOLDER_NAME = "json_data"
+JSON_DATA_FILE_NAME = "json_data"
 
-ENTITY_QUERY = {"sort": "-createdDate"}
+ATTRIBUTE_QUERY = {"active-eq": True}
 DEFAULT_GROUP_NAME = "~ no group"
 
 # --------- Generate Python Code ---------
